@@ -1,10 +1,8 @@
 /****h* ROBODoc/Directory
  * NAME
  *   This module contains function to scan a directory tree and to
- *   create a RB_Directory structure.
- *
- *   Most of the OS dependend parts of ROBODoc can be found in this
- *   module..
+ *   create a RB_Directory structure.  Most of the OS dependend parts
+ *   of ROBODoc can be found in this module..
  *
  *   This module currently works under:
  *   o Cygwin   http://cygwin.com 
@@ -16,7 +14,7 @@
  * AUTHOR 
  *   Frans Slothouber
  *****
- * $Id: directory.c,v 1.26 2003/12/30 17:39:36 gumpu Exp $
+ * $Id: directory.c,v 1.32 2004/09/25 18:12:53 gumpu Exp $
  */
 
 #include <assert.h>
@@ -275,7 +273,13 @@ RB_Get_RB_Directory( char *arg_rootpath_name, char* arg_docroot_name )
     }
 
     RB_Fill_Directory( rb_directory, rb_directory->first_path, doc_path );
+    if ( ( RB_Number_Of_Filenames( rb_directory ) > 0 ) &&
+         ( RB_Number_Of_Paths( rb_directory ) > 0 ) ) {
+        RB_SortDirectory( rb_directory );
+    } else {
 
+        RB_Panic( "No files found! (Or all were filtered out)\n" );
+    }
     return rb_directory;
 }
 
@@ -456,7 +460,7 @@ RB_Fill_Directory( struct RB_Directory *arg_rb_directory,
     RB_Say( "Scanning %s\n", arg_path->name );
     a_dirstream = opendir( arg_path->name );
 
-    if ( a_direntry )
+    if ( a_dirstream )
     {
         T_RB_FileType       file_type;
 
@@ -533,8 +537,6 @@ RB_Fill_Directory( struct RB_Directory *arg_rb_directory,
 }
 
 
-
-
 /****f* Directory/RB_Is_Source_File
  * NAME
  *   RB_Is_Source_File -- Is a file a sourcefile?
@@ -552,7 +554,7 @@ RB_Is_Source_File( struct RB_Path *path, char *filename )
 {
     int                 is_source = 1;
 
-    if ( RB_To_Be_Skipped( filename ) )
+    if ( RB_Not_Accepted( filename ) )
     {
         /* File needs to be skipped based on it's name */
         is_source = 0;
@@ -654,6 +656,45 @@ RB_To_Be_Skipped( char *filename )
 /******/
 
 
+/****f* Directory/RB_Not_Accepted
+ * FUNCTION
+ *   Test if a file should be skipped, 
+ *   because it does not match a pattern in "accept files:"
+ *
+ *   This test is done based on the wildcard expressions specified
+ *   in configuration.accept_files.
+ *
+ * INPUTS
+ *   o filename -- the name of the file
+ * SOURCE
+ */
+
+int
+RB_Not_Accepted( char *filename )
+{
+    unsigned int        i;
+    int                 skip = FALSE;
+
+    skip = RB_To_Be_Skipped(filename);
+
+    if(!skip && configuration.accept_files.number > 0) {
+        skip = TRUE;
+        for ( i = 0; i < configuration.accept_files.number; ++i )
+        {
+            if ( RB_Match( filename, configuration.accept_files.names[ i ] ) )
+            {
+                RB_Say( "accept >%s< with >%s<\n", filename, configuration.accept_files.names[ i ] );
+                skip = FALSE;
+                break;
+            }
+        }
+    }
+    return skip;
+}
+
+/******/
+
+
 /****f* Directory/RB_Get_FileName
  * NAME
  *   RB_Get_PathName -- extract the file name 
@@ -819,3 +860,109 @@ RB_Is_PathCharacter( int c )
 
 /******/
 
+
+/* Sort the files and paths */
+
+unsigned int RB_Number_Of_Filenames( struct RB_Directory *arg_rb_directory )
+{
+    unsigned int number_of_filenames = 0;
+    unsigned int i = 0;
+    struct RB_Filename *rb_filename = NULL;
+
+    for ( rb_filename = arg_rb_directory->first;
+          rb_filename;
+          ++number_of_filenames, rb_filename = rb_filename->next ) 
+    {
+        /* Empty */
+    }
+    return number_of_filenames;
+}
+
+
+unsigned int RB_Number_Of_Paths( struct RB_Directory *arg_rb_directory )
+{
+    unsigned int number_of_paths = 0;
+    unsigned int i = 0;
+    struct RB_Path *rb_path = NULL;
+
+    for ( rb_path = arg_rb_directory->first_path;
+          rb_path;
+          ++number_of_paths, rb_path = rb_path->next ) 
+    {
+        /* Empty */
+    }
+    return number_of_paths;
+}
+
+int RB_Path_Compare( void* p1, void* p2 )
+{
+    struct RB_Path *path_1 = p1;
+    struct RB_Path *path_2 = p2;
+    return RB_Str_Case_Cmp( path_1->name, path_2->name );
+}
+
+int RB_Filename_Compare( void* p1, void* p2 )
+{
+    struct RB_Filename *filename_1 = p1;
+    struct RB_Filename *filename_2 = p2;
+    return RB_Str_Case_Cmp( filename_1->name, filename_2->name );
+}
+
+
+void RB_SortDirectory( struct RB_Directory *arg_rb_directory )
+{
+    unsigned int number_of_filenames = RB_Number_Of_Filenames( arg_rb_directory );
+    unsigned int number_of_paths     = RB_Number_Of_Paths( arg_rb_directory );
+    unsigned int i;
+    struct RB_Path *rb_path = NULL;
+    struct RB_Filename *rb_filename = NULL;
+    struct RB_Path** paths = NULL;
+    struct RB_Filename** filenames = NULL;
+
+    assert( number_of_filenames > 0 );
+    assert( number_of_paths > 0 );
+    paths = calloc( number_of_paths, sizeof( struct RB_Path* ) );
+    filenames = calloc( number_of_filenames, sizeof( struct RB_Filename* ) );
+
+
+    RB_Say( "Sorting Directory\n" );
+    for( i = 0, rb_path = arg_rb_directory->first_path; 
+         rb_path;
+         rb_path = rb_path->next ) 
+    {
+        assert( i < number_of_paths );
+        paths[ i ] = rb_path;
+        ++i;
+    }
+
+   for( i = 0, rb_filename = arg_rb_directory->first; 
+         rb_filename;
+         rb_filename = rb_filename->next ) 
+    {
+        assert( i < number_of_filenames );
+        filenames[ i ] = rb_filename;
+        ++i;
+    }
+
+    RB_QuickSort( paths, 0, number_of_paths - 1, RB_Path_Compare );
+    RB_QuickSort( filenames, 0, number_of_filenames - 1, RB_Filename_Compare );
+
+    for( i = 0; i < number_of_paths - 1; ++i )
+    {
+        paths[ i ]->next = paths[ i + 1 ];
+    }
+    paths[ number_of_paths - 1 ]->next = NULL;
+    arg_rb_directory->first_path = paths[ 0 ];
+
+    for( i = 0; i < number_of_filenames - 1; ++i )
+    {
+        filenames[ i ]->next = filenames[ i + 1 ];
+    }
+    filenames[ number_of_filenames - 1 ]->next = NULL;
+    arg_rb_directory->first = filenames[ 0 ];
+    arg_rb_directory->last  = filenames[ number_of_filenames - 1 ];
+
+    free( paths );
+    free( filenames );
+}
+
