@@ -6,7 +6,7 @@
  * AUTHOR
  *   PetteriK
  *********
- * $Id: folds.c,v 1.14 2003/12/30 17:39:36 gumpu Exp $
+ * $Id: folds.c,v 1.15 2004/05/18 20:45:12 gumpu Exp $
  */
 
 #include <stdlib.h>
@@ -117,7 +117,7 @@ int                 fold_id = 0;
 
 char               *
 RB_Fold( char *line, char **doc_name, FILE ** dest_doc, char *extension,
-         char *charset )
+         char *charset, char* item_name )
 {
     if ( course_of_action & DO_FOLDS )
     {
@@ -167,7 +167,7 @@ RB_Fold( char *line, char **doc_name, FILE ** dest_doc, char *extension,
             {
                 fputc( ' ', old_file );
             }
-            RB_Generate_Link( old_file, name, new_doc_name, foldname,
+            Generate_Link( old_file, name, new_doc_name, foldname,
                               number_string );
             fputc( '\n', old_file );
             printf( "closing %s\n", name );
@@ -176,7 +176,7 @@ RB_Fold( char *line, char **doc_name, FILE ** dest_doc, char *extension,
             /* and the doc start */
             RB_Generate_Doc_Start( new_file, "dummy", new_doc_name, 0,
                                    new_doc_name, charset );
-            RB_Generate_Item_Begin( new_file );
+            RB_Generate_Item_Begin( new_file, item_name );
             ++fold_id;
             ++fold;
             return foldname;
@@ -184,7 +184,7 @@ RB_Fold( char *line, char **doc_name, FILE ** dest_doc, char *extension,
         else if ( RB_Check_Fold_End( line ) )
         {
             --fold;
-            RB_Generate_Item_End( *dest_doc );
+            RB_Generate_Item_End( *dest_doc, item_name );
             RB_Generate_Doc_End( *dest_doc, *doc_name );
             printf( "closing %s\n", *doc_name );
             free( *doc_name );
