@@ -39,10 +39,8 @@ extern fold_mark_t  fold_start_markers[];
 extern fold_mark_t  fold_end_markers[];
 extern long         extra_flags;
 extern int          fold;
-
-char               *RB_Fold( char *line, char **doc_name, FILE ** dest_doc,
-                             char *extension, char *charset );
-
+char               *RB_Fold( char *line, char **doc_name, FILE ** dest_doc, 
+                             char *extension, char *charset, char* item_name );
 char                RB_Inside_Fold( void );
 void                RB_Enter_Fold( void );
 void                RB_Exit_Fold( void );
