@@ -16,6 +16,7 @@
 #include "robodoc.h"
 #include "headers.h"
 #include "globals.h"
+#include "roboconfig.h"
 #include "util.h"
 
 #define NO_MARKER_LOCKED -1
@@ -64,6 +65,8 @@ char               *header_markers[] = {
     "$!****",                   /* 19  DCL */
     "'****",                    /* 20  VB, LotusScript */
     ".****",                    /* 21  DB/C */
+    "\\ ****",                  /* 22  Forth */
+    "<!-- ****",                /* 23  XML */
     NULL
 };
 
@@ -121,6 +124,7 @@ char               *remark_markers[] = {
     "$!",                       /* 16 DCL */
     "'*",                       /* 17 VB */
     ".*",                       /* 18 DB/C */
+    "\\",                       /* 19 Forth */
     NULL
 };
 
@@ -166,6 +170,8 @@ char               *end_markers[] = {
     "$!***",                    /* 20 DCL */
     "'***",                     /* 21 VB, LotusScript */
     ".***",                     /* 22 DB/C */
+    "\\ ***",                   /* 23 Forth */
+    "<!-- ***",                 /* 24 XML */
     NULL
 };
 
@@ -195,13 +201,14 @@ char               *end_remark_markers[] = {
     ( char * ) NULL,            /* 12  Tcl/Tk */
     ( char * ) NULL,            /* 13  COBOL */
     ( char * ) NULL,            /* 14  Occam */
-    "-->",                      /* 15  HTML */
+    "-->",                      /* 15  HTML  & XML */
     "--->",                     /* 16  HTML */
     ( char * ) NULL,            /* 17  GNU Assembler */
     ( char * ) NULL,            /* 18  FORTRAN 90 !! */
     ( char * ) NULL,            /* 19  FORTRAN 90 !*/
     ( char * ) NULL,            /* 20  VB */
-    ( char * ) NULL             /* 21  Aspen Plus */
+    ( char * ) NULL,            /* 21  Aspen Plus */
+    ( char * ) NULL             /* 22  Forth */
 };
 
 /****/
@@ -225,66 +232,6 @@ char               *robo_end[] = { "/*ROBODOC_END*", "*ROBODOC_END*", NULL };
 /****/
 
 
-/*x**v* Headers/RB_header_typenames
- * NAME
- *   RB_header_typename
- * FUNCTION
- *   Handy table to translate a header type number (see RB_header_types)
- *   to an ascii string.
- *
- *   OBSOLETE
- *
- *****
- */
-
-char               *RB_header_type_names[] = {
-    "none",
-    "main",
-    "generic",
-    "function",
-    "struct",
-    "class",
-    "method",
-    "constant",
-    "variable",
-    "blank",
-    "procedure",
-    "type",
-    "exception"
-};
-
-/*x**v* Headers/RB_internal_header_type_names
- * NAME
- *   RB_internal_header_type_names
- * FUNCTION
- *   Handy table to translate a header type number (see RB_header_types)
- *   to an ascii string.
- * NOTES
- *   (RB_header.type & ~INTERNAL_HEADER) should be used as the index
- *   into this array.  
- *    
- *    OBSOLETE
- *
- *****
- */
-
-char               *RB_internal_header_type_names[] = {
-    "none",
-    "main (internal)",
-    "generic (internal)",
-    "function (internal)",
-    "struct (internal)",
-    "class (internal)",
-    "method (internal)",
-    "constant (internal)",
-    "variable (internal)",
-    "blank (internal)",
-    "procedure (internal)",
-    "type (internal)",
-    "exception (internal)"
-};
-
-
 /***f* Headers/RB_Is_Begin_Marker
  * FUNCTION
  *   Scan a line and see if any of the begin of a header markers 
@@ -320,9 +267,10 @@ RB_Is_Begin_Marker( char *cur_line, char **type )
        )
     {
         for ( marker = 0;
-              ( ( cur_mchar = header_markers[ marker ] ) != NULL )
-              && !found; marker++ )
+              ( marker < configuration.header_markers.number ) && !found;
+              marker++ )
         {
+            cur_mchar = configuration.header_markers.names[ marker ];
             for ( found = TRUE, cur_char = RB_Skip_Whitespace( cur_line );
                   *cur_mchar && *cur_char && found; cur_mchar++, cur_char++ )
             {
@@ -334,7 +282,7 @@ RB_Is_Begin_Marker( char *cur_line, char **type )
     else if ( ( course_of_action & DO_LOCKHEADER ) &&
               ( locked_header_marker != NO_MARKER_LOCKED ) )
     {
-        cur_mchar = header_markers[ locked_header_marker ];
+        cur_mchar = configuration.header_markers.names[ locked_header_marker ];
         for ( found = TRUE, cur_char = RB_Skip_Whitespace( cur_line );
                 *cur_mchar && *cur_char && found; cur_mchar++, cur_char++ )
         {
@@ -346,6 +294,7 @@ RB_Is_Begin_Marker( char *cur_line, char **type )
     }
     else
     {
+        /* Obsolete */
         /* use robo_header instead if -rh command line switch */
         if ( ( ( cur_mchar = robo_header ) != NULL ) )
         {
@@ -363,19 +312,34 @@ RB_Is_Begin_Marker( char *cur_line, char **type )
     if ( found && cur_char )
     {
         *type = cur_char;
-        /* Do some additional checks to make sure it is
-         * really a begin marker and not an end marker.
-         */
-        found = FALSE;
-        /* It should contain a '/' and some non * characters. */
-        for ( ; *cur_char && *cur_char != '/'; ++cur_char )
+        ++cur_char;
+        if ( *cur_char == '*' )
         {
-            if ( *cur_char != '*' )
+            ++cur_char;
+            found = isspace( *cur_char );
+        }
+        else if ( *(cur_char + 1) == '*') 
+        {
+            ++cur_char;
+            ++cur_char;
+            found = isspace( *cur_char );
+        }
+        else
+        {
+            found = FALSE;
+        }
+        if ( found )
+        {
+            found = FALSE;
+            /* It should contain some non * characters. */
+            for ( ; *cur_char; ++cur_char )
             {
-                found = TRUE;
+                if ( isalnum( *cur_char )  )
+                {
+                    found = TRUE;
+                }
             }
         }
-        found = found && ( *cur_char == '/' ) && ( *( cur_char - 1 ) != '*' );
     }
 
     if ( found && 
@@ -384,7 +348,7 @@ RB_Is_Begin_Marker( char *cur_line, char **type )
     {
         assert( marker != NO_MARKER );
         locked_header_marker = marker - 1;
-        RB_Say( "header marker locked on %s\n", header_markers[ locked_header_marker ] );
+        RB_Say( "header marker locked on %s\n", configuration.header_markers.names[ locked_header_marker ] );
     }
     return found;
 }
@@ -427,11 +391,34 @@ RB_Is_End_Marker( char *cur_line )
        )
     {
         for ( marker = 0;
-              ( ( cur_mchar = end_markers[marker] ) != NULL ) && !found;
+              ( marker < configuration.end_markers.number ) && !found;
               marker++ )
         {
-            for ( found = TRUE, cur_char = RB_Skip_Whitespace( cur_line );
-                  *cur_mchar && *cur_char && found; cur_mchar++, cur_char++ )
+            cur_mchar = configuration.end_markers.names[marker];
+            cur_char = RB_Skip_Whitespace( cur_line );
+            if ( *cur_char ) 
+            {
+                for ( found = TRUE;
+                        *cur_mchar && *cur_char && found; cur_mchar++, cur_char++ )
+                {
+                    if ( tolower( *cur_mchar ) != tolower( *cur_char ) )
+                    {
+                        found = FALSE;
+                    }
+                }
+            }
+        }
+    }
+    else if ( ( course_of_action & DO_LOCKHEADER ) &&
+              ( locked_end_marker != NO_MARKER_LOCKED ) )
+    {
+        cur_mchar = configuration.end_markers.names[ locked_end_marker ];
+        cur_char = RB_Skip_Whitespace( cur_line );
+        if ( *cur_char ) 
+        {
+            for ( found = TRUE;
+                  *cur_mchar && *cur_char && found; 
+                  cur_mchar++, cur_char++ )
             {
                 if ( tolower( *cur_mchar ) != tolower( *cur_char ) )
                 {
@@ -440,22 +427,9 @@ RB_Is_End_Marker( char *cur_line )
             }
         }
     }
-    else if ( ( course_of_action & DO_LOCKHEADER ) &&
-              ( locked_end_marker != NO_MARKER_LOCKED ) )
-    {
-        cur_mchar = end_markers[ locked_end_marker ];
-        for ( found = TRUE, cur_char = RB_Skip_Whitespace( cur_line );
-              *cur_mchar && *cur_char && found; 
-              cur_mchar++, cur_char++ )
-        {
-            if ( tolower( *cur_mchar ) != tolower( *cur_char ) )
-            {
-                found = FALSE;
-            }
-        }
-    }
     else
     {
+        /* TODO OBSOLETE */
         /*use robo_end if using -rh switch */
         for ( marker = 0;
               ( ( cur_mchar = robo_end[marker] ) != NULL ) && !found;
@@ -475,8 +449,9 @@ RB_Is_End_Marker( char *cur_line )
     /* Locking on end markers does not work at the moment,
      * because there can be more than one end marker for
      * a given language.
+     * TODO 
      */
-#if 0
+#if 0    
     if ( found &&
          ( course_of_action & DO_LOCKHEADER ) && 
          ( locked_end_marker == NO_MARKER_LOCKED ) ) 
@@ -529,9 +504,9 @@ int RB_Has_Remark_Marker( char* lline_buffer )
            !( course_of_action & DO_LOCKHEADER ) 
        )
     {
-        for ( marker = 0; remark_markers[ marker ] != NULL; marker++ )
+        for ( marker = 0; marker < configuration.remark_markers.number; marker++ )
         {
-            if ( RB_Str_Case_Cmp( lline_buffer, remark_markers[ marker ] ) == 0 ) 
+            if ( RB_Str_Case_Cmp( lline_buffer, configuration.remark_markers.names[ marker ] ) == 0 ) 
             {
                 marker_found = marker;
                 found = TRUE;
@@ -540,7 +515,7 @@ int RB_Has_Remark_Marker( char* lline_buffer )
     }
     else
     {
-        if ( RB_Str_Case_Cmp( lline_buffer, remark_markers[ locked_remark_marker ] ) == 0 ) 
+        if ( RB_Str_Case_Cmp( lline_buffer, configuration.remark_markers.names[ locked_remark_marker ] ) == 0 ) 
         {
             marker_found = marker;
             found = TRUE;
@@ -555,7 +530,7 @@ int RB_Has_Remark_Marker( char* lline_buffer )
     {
         assert( marker_found != NO_MARKER );
         locked_remark_marker = marker_found;
-        RB_Say( "remark marker locked on %s\n", remark_markers[ locked_remark_marker ] );
+        RB_Say( "remark marker locked on %s\n", configuration.remark_markers.names[ locked_remark_marker ] );
     }
 
     /* Restore the space we replaced with a '\0' */
@@ -591,11 +566,12 @@ RB_Skip_Remark_Marker( char *lline_buffer )
     found = FALSE;
     cur_char = NULL;
     for ( marker = 0;
-          ( ( cur_mchar = remark_markers[ marker ] ) != NULL ) && !found;
-          marker++ )
+         ( marker < configuration.remark_markers.number ) && !found;
+         marker++ )
     {
+        cur_mchar = configuration.remark_markers.names[ marker ];
         for ( found = TRUE, cur_char = lline_buffer;
-                *cur_mchar && *cur_char && found; cur_mchar++, cur_char++ )
+              *cur_mchar && *cur_char && found; cur_mchar++, cur_char++ )
         {
             if ( tolower( *cur_mchar ) != tolower( *cur_char ) )
             {
@@ -608,10 +584,15 @@ RB_Skip_Remark_Marker( char *lline_buffer )
 
 /**************/
 
-
+/* TODO Documentation */
 void RB_Header_Lock_Reset( void )
 {
     locked_header_marker = NO_MARKER_LOCKED;
     locked_end_marker    = NO_MARKER_LOCKED;
+}
+
+void RB_Item_Lock_Reset( void )
+{
     locked_remark_marker = NO_MARKER_LOCKED;
 }
+
