@@ -10,7 +10,7 @@
  *   2003-02-03   Frans Slothouber  Refactoring
  *   ????-??-??   Frans Slothouber  V1.0
  *******
- * $Id: html_generator.c,v 1.43 2003/12/30 17:39:36 gumpu Exp $
+ * $Id: html_generator.c,v 1.51 2004/09/25 18:12:53 gumpu Exp $
  */
 
 #include <stdio.h>
@@ -33,6 +33,7 @@
 #include "document.h"
 #include "directory.h"
 #include "path.h"
+#include "part.h"
 
 #ifdef DMALLOC
 #include <dmalloc.h>
@@ -40,12 +41,10 @@
 
 char               *css_name = NULL;
 
-
 void RB_HTML_Generate_String( FILE * dest_doc, char *a_string );
 
 
-
-
+/* TODO  Documentation */
 static void RB_HTML_Generate_Source_Tree_Entry(
         FILE *dest_doc, 
         char *dest_name,
@@ -64,13 +63,15 @@ static void RB_HTML_Generate_Source_Tree_Entry(
         if ( cur_filename->path == parent_path )
         {
             char               *r = 0;
-
-            r = RB_HTML_RelativeAddress( dest_name, cur_filename->link->file_name );
-            fprintf( dest_doc, "<li>\n" );
-            fprintf( dest_doc, "<a href=\"%s#%s\"><tt>\n", r,
-                    cur_filename->link->label_name );
-            RB_HTML_Generate_String( dest_doc, cur_filename->name );
-            fprintf( dest_doc, "</tt></a></li>\n" );
+            if ( cur_filename->link ) 
+            {
+                r = RB_HTML_RelativeAddress( dest_name, cur_filename->link->file_name );
+                fprintf( dest_doc, "<li>\n" );
+                fprintf( dest_doc, "<a href=\"%s#%s\"><tt>\n", r,
+                        cur_filename->link->label_name );
+                RB_HTML_Generate_String( dest_doc, cur_filename->name );
+                fprintf( dest_doc, "</tt></a></li>\n" );
+            }
         }
     }
     for( cur_path = srctree->first_path;
@@ -89,6 +90,7 @@ static void RB_HTML_Generate_Source_Tree_Entry(
 }
 
 
+/* TODO  Documentation */
 void RB_HTML_Generate_Source_Tree( 
         FILE *dest_doc, 
         char *dest_name,
@@ -96,11 +98,7 @@ void RB_HTML_Generate_Source_Tree(
 {
     struct RB_Directory* srctree;
     srctree = document->srctree;
-    RB_HTML_Generate_Source_Tree_Entry(
-                dest_doc,
-                dest_name,
-                NULL,
-                srctree );
+    RB_HTML_Generate_Source_Tree_Entry( dest_doc, dest_name, NULL, srctree );
 }
 
 
@@ -120,7 +118,7 @@ void RB_HTML_Generate_Source_Tree(
  */
 
 void
-RB_HTML_Generate_String( FILE * dest_doc, char *a_string )
+RB_HTML_Generate_String( FILE* dest_doc, char* a_string )
 {
     int                 i;
     int                 l = strlen( a_string );
@@ -143,8 +141,8 @@ RB_HTML_Generate_String( FILE * dest_doc, char *a_string )
  * SYNOPSIS
  *   void RB_HTML_Generate_False_Link( FILE * dest_doc, char *name )
  * INPUTS
- *   dest_doc -- the file the representation is written to.
- *   name     -- the word.
+ *   * dest_doc -- the file the representation is written to.
+ *   * name     -- the word.
  * SOURCE
  */
 
@@ -154,6 +152,7 @@ RB_HTML_Generate_False_Link( FILE * dest_doc, char *name )
     fprintf( dest_doc, "<strong>" );
     RB_HTML_Generate_String( dest_doc, name );
     fprintf( dest_doc, "</strong>" );
+
 }
 
 /*******/
@@ -174,7 +173,7 @@ RB_HTML_Generate_False_Link( FILE * dest_doc, char *name )
  */
 
 int
-RB_HTML_Generate_Extra( FILE * dest_doc, int item_type, char *cur_char )
+RB_HTML_Generate_Extra( FILE * dest_doc, enum ItemType item_type, char *cur_char )
 {
     char                link[1024];
     int                 res = 0, prev;
@@ -300,16 +299,39 @@ RB_HTML_Generate_Extra( FILE * dest_doc, int item_type, char *cur_char )
 /******/
 
 
+
 void
-RB_HTML_Generate_Item_Begin( FILE * dest_doc )
+RB_HTML_Generate_Item_Name( FILE * dest_doc, char *name )
 {
-    fprintf( dest_doc, "<pre>" );
+    char* classname = RB_StrDup( name );
+    char* c = classname;
+
+    fprintf( dest_doc, "<p><strong>" );
+    RB_HTML_Generate_String( dest_doc, name );
+    fprintf( dest_doc, "</strong></p>\n" );
+
+    for( ;*c; ++c ) 
+    {
+        if ( isspace( *c ) )
+        {
+            *c = '_';
+        }
+    }
+    fprintf( dest_doc, "<span class=\"%s\">", classname );
+
+    free( classname );
 }
 
 void
-RB_HTML_Generate_Item_End( FILE * dest_doc )
+RB_HTML_Generate_Item_Begin( FILE * dest_doc, char* name )
 {
-    fprintf( dest_doc, "</pre>" );
+
+}
+
+void
+RB_HTML_Generate_Item_End( FILE * dest_doc, char* name )
+{
+    fprintf( dest_doc, "</span>" );
 }
 
 void
@@ -318,20 +340,21 @@ RB_HTML_Generate_TOC_1( FILE * dest_doc, struct RB_header *header )
     assert( 0 );
 }
 
-int                 sectiontoc_counters[MAX_SECTION_DEPTH];
 
 
+
+int                 sectiontoc_counters[MAX_SECTION_DEPTH];
+
 /****f* HTML_Generator/RB_HTML_Generate_TOC_Section
  * FUNCTION
  *   Create a table of contents based on the hierarchy of
  *   the headers starting for a particular point in this
  *   hierarchy (the parent).
  * SYNOPSIS
- *   void
- *   RB_HTML_Generate_TOC_Section( FILE * dest_doc, char *dest_name,
- *                                 struct RB_header *parent,
- *                                 struct RB_header **headers, int count,
- *                                 int depth )
+ *   void*   RB_HTML_Generate_TOC_Section( FILE * dest_doc, char *dest_name,
+ *   struct RB_header *parent,
+ *   struct RB_header **headers, int count,
+ *   int depth )
  * INPUTS
  *   o dest_doc  -- the file to write to.
  *   o dest_name -- the name of this file.
@@ -494,15 +517,6 @@ RB_HTML_Generate_TOC_2( FILE * dest_doc, struct RB_header **headers,
 
 
 
-void
-RB_HTML_Generate_Item_Name( FILE * dest_doc, char *name )
-{
-    fprintf( dest_doc, "<p><strong>" );
-    RB_HTML_Generate_String( dest_doc, name );
-    fprintf( dest_doc, "</strong></p>\n" );
-}
-
-
 /****f* HTML_Generator/RB_HTML_Generate_Label
  * FUNCTION
  *   Generate a label (name) that can be refered too.
@@ -550,7 +564,9 @@ RB_HTML_Generate_Label( FILE * dest_doc, char *name )
 int                 section_counters[MAX_SECTION_DEPTH];
 
 
-void
+/* TODO Documentation */
+
+    void
 RB_HTML_Generate_BeginSection( FILE * dest_doc, int depth, char *name )
 {
     int                 i;
@@ -562,25 +578,25 @@ RB_HTML_Generate_BeginSection( FILE * dest_doc, int depth, char *name )
     }
     switch ( depth )
     {
-    case 1:
-    case 2:
-    case 3:
-    case 4:
-    case 5:
-    case 6:
-    case 7:
-        fprintf( dest_doc, "<h%d>", depth );
-        for ( i = 1; i <= depth; ++i )
-        {
-            fprintf( dest_doc, "%d.", section_counters[i] );
-        }
-        fprintf( dest_doc, "  " );
-        RB_HTML_Generate_String( dest_doc, name );
-        fprintf( dest_doc, "  </h%d>\n", depth );
-        break;
-    default:
-        /* too deep, don't do anything. */
-        assert( 0 );
+        case 1:
+        case 2:
+        case 3:
+        case 4:
+        case 5:
+        case 6:
+        case 7:
+            fprintf( dest_doc, "<h%d>", depth );
+            for ( i = 1; i <= depth; ++i )
+            {
+                fprintf( dest_doc, "%d.", section_counters[i] );
+            }
+            fprintf( dest_doc, "  " );
+            RB_HTML_Generate_String( dest_doc, name );
+            fprintf( dest_doc, "  </h%d>\n", depth );
+            break;
+        default:
+            /* too deep, don't do anything. */
+            assert( 0 );
     }
 }
 
@@ -603,10 +619,10 @@ RB_HTML_Get_Default_Extension( void )
  * FUNCTION
  *   Generate the first part of a HTML document.
  *   As far as ROBODoc is concerned a HTML document
- *   consists of three parts.
- *   (1) The start of a document
- *   (2) The body of a document
- *   (3) The end of a document
+ *   consists of three parts:
+ *   * The start of a document
+ *   * The body of a document
+ *   * The end of a document
  * INPUTS
  *   o dest_doc  --  the output file.
  *   o src_name  --  The file or directoryname from which 
@@ -624,58 +640,70 @@ void
 RB_HTML_Generate_Doc_Start( FILE * dest_doc, char *src_name, char *name,
                             char toc, char *dest_name, char *charset )
 {
-    /* Append document type and title */
-    fprintf( dest_doc,
-             "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">\n" );
-    fprintf( dest_doc, "<html>\n<head>\n" );
-    RB_InsertCSS( dest_doc, dest_name );
 
-    if ( charset )
+    if ( course_of_action & DO_HEADLESS ) 
     {
+        /* The user wants a headless document, so we skip everything
+         * upto and until <BODY>
+         */
+    }
+    else
+    {
+        /* Append document type and title */
         fprintf( dest_doc,
-                 "<meta http-equiv=\"Content-type\" content=\"text/html; charset=%s\">\n",
-                 charset );
-    }
+                "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">\n" );
+        fprintf( dest_doc, "<html>\n<head>\n" );
+        RB_InsertCSS( dest_doc, dest_name );
 
-    fprintf( dest_doc, "<title>%s</title>\n", name );
-
-    /* append SGML-comment with document- and copyright-info. This code
-     * ensures that every line has an own comment to avoid problems with
-     * buggy browsers */
-    fprintf( dest_doc, "<!-- Source: %s -->\n", src_name );
-    {
-        static const char   copyright_text[]
-            = COMMENT_ROBODOC COMMENT_COPYRIGHT;
-        size_t              i = 0;
-        char                previous_char = '\n';
-        char                current_char = copyright_text[i];
-
-        while ( current_char )
+        if ( charset )
         {
-            if ( previous_char == '\n' )
-            {
-                fprintf( dest_doc, "<!-- " );
-            }
-            if ( current_char == '\n' )
-            {
-                fprintf( dest_doc, " -->" );
-            }
-            else if ( ( current_char == '-' ) && ( previous_char == '-' ) )
-            {
-                /* avoid "--" inside SGML-comment, and use "-_" instead; this
-                 * looks a bit strange, but one should still be able to figure
-                 * out what is meant when reading the output */
-                current_char = '_';
-            }
-            fputc( current_char, dest_doc );
-            i += 1;
-            previous_char = current_char;
-            current_char = copyright_text[i];
+            fprintf( dest_doc,
+                    "<meta http-equiv=\"Content-type\" content=\"text/html; charset=%s\">\n",
+                    charset );
         }
-    }
 
-    /* append heading and start list of links to functions */
-    fprintf( dest_doc, "</head>\n<body bgcolor=\"#FFFFFF\">\n" );
+        fprintf( dest_doc, "<title>%s</title>\n", name );
+
+        /* append SGML-comment with document- and copyright-info. This code
+         * ensures that every line has an own comment to avoid problems with
+         * buggy browsers */
+        fprintf( dest_doc, "<!-- Source: %s -->\n", src_name );
+        {
+            static const char   copyright_text[]
+                = COMMENT_ROBODOC COMMENT_COPYRIGHT;
+            size_t              i = 0;
+            char                previous_char = '\n';
+            char                current_char = copyright_text[i];
+
+            while ( current_char )
+            {
+                if ( previous_char == '\n' )
+                {
+                    fprintf( dest_doc, "<!-- " );
+                }
+                if ( current_char == '\n' )
+                {
+                    fprintf( dest_doc, " -->" );
+                }
+                else if ( ( current_char == '-' ) && ( previous_char == '-' ) )
+                {
+                    /* avoid "--" inside SGML-comment, and use "-_" instead; this
+                     * looks a bit strange, but one should still be able to figure
+                     * out what is meant when reading the output */
+                    current_char = '_';
+                }
+                fputc( current_char, dest_doc );
+                i += 1;
+                previous_char = current_char;
+                current_char = copyright_text[i];
+            }
+        }
+
+        /* append heading and start list of links to functions */
+        fprintf( dest_doc, "</head>\n" );
+        fprintf( dest_doc, "<body bgcolor=\"#FFFFFF\">\n" );
+    } 
+
 
     /* handle folds bit differently */
     if ( ( extra_flags & FOLD ) && RB_Inside_Fold(  ) )
@@ -691,9 +719,12 @@ RB_HTML_Generate_Doc_Start( FILE * dest_doc, char *src_name, char *name,
     else
     {
         /* normal case, no folding business */
+        /* TODO This should be done with
+         * RB_Generate_Label() 
+         */
         fprintf( dest_doc,
                  "<a name=\"%s\">Generated from %s</a> with <a href=\"http://sourceforge.net/projects/robodoc/\">ROBODoc</a> v%s on ",
-                 src_name, src_name, VERSION );
+                 "robo_top_of_doc", src_name, VERSION );
         RB_TimeStamp( dest_doc );
         fprintf( dest_doc, "<br />\n" );
     }
@@ -714,25 +745,83 @@ RB_HTML_Generate_Doc_Start( FILE * dest_doc, char *src_name, char *name,
 void
 RB_HTML_Generate_Doc_End( FILE * dest_doc, char *name )
 {
-    fprintf( dest_doc, "</body>\n</html>\n" );
+    if ( course_of_action & DO_FOOTLESS ) 
+    {
+        /* The user does not want the foot of the
+         * document.
+         */
+    }
+    else
+    {
+        fprintf( dest_doc, "</body>\n</html>\n" );
+    }
 }
 
 
-/* TODO */
+void RB_HTML_Generate_Nav_Bar( struct RB_Document* document, FILE* current_doc, struct RB_header *current_header )
+{
+    char* current_filename = NULL;
+    char* target_filename = NULL;
+    char* label = NULL;
+    current_filename = RB_Get_FullDocname( current_header->owner->filename );
+    target_filename  = RB_Get_FullDocname( current_header->owner->filename );
+    label            = Get_Fullname( current_header->owner->filename );
+    /* The navigation bar */
+    fprintf( current_doc, "<p>" );
+    fprintf( current_doc, "[" );
+    //            [Top][Parent][Index]
+    RB_HTML_Generate_Link( current_doc, current_filename, NULL, "robo_top_of_doc", "top", 0 );
+    fprintf( current_doc, "]" );
+    if ( current_header->parent ) 
+    {
+        fprintf( current_doc, "[" );
+        target_filename = RB_Get_FullDocname( current_header->parent->owner->filename );
+        label = current_header->parent->unique_name;
+        RB_HTML_Generate_Link( current_doc, current_filename, target_filename, label, "parent", 0 );
+        fprintf( current_doc, "]" );
+    }
+    if ( ( course_of_action & DO_INDEX ) &&
+         ( course_of_action & DO_MULTIDOC ) )
+    {
+        fprintf( current_doc, "[" );
+            target_filename = RB_Get_SubIndex_FileName( 
+                        document->docroot->name, 
+                        document->extension, 
+                        current_header->htype
+                        );
+        RB_HTML_Generate_Link( current_doc, current_filename, target_filename, "top", "index", 0 );
+        free( target_filename );
+        fprintf( current_doc, "]" );
+    }
+    fprintf( current_doc, "</p>\n" );
+}
+
+
+/* TODO Documentation */
 /*x**if* HTML_Generator/RB_HTML_Generate_Header_Start
  * NAME
  *   RB_HTML_Generate_Header_Start --
  ******
- * OBSOLETE
  */
 
 void
-RB_HTML_Generate_Header_Start( FILE * dest_doc, struct RB_header *cur_header )
+RB_HTML_Generate_Header_Start( FILE* dest_doc, struct RB_header *cur_header )
 {
+    struct RB_HeaderType* header_type;
+
     if ( cur_header->name && cur_header->function_name )
     {
         fprintf( dest_doc, "<hr />\n" );
         fprintf( dest_doc, "\n<h2><a name=\"%s\">", cur_header->unique_name );
+
+         header_type = RB_FindHeaderType( cur_header->htype->typeCharacter );
+         if ( header_type ) 
+         {
+             fprintf( dest_doc, "[" );
+             RB_HTML_Generate_String( dest_doc, header_type->indexName );
+             fprintf( dest_doc, "]" );
+         }
+
         RB_HTML_Generate_Label( dest_doc, cur_header->name );
         RB_HTML_Generate_String( dest_doc, cur_header->name );
         fprintf( dest_doc, "</a></h2>\n\n" );
@@ -760,9 +849,10 @@ RB_HTML_Generate_Header_End( FILE * dest_doc, struct RB_header *cur_header )
  *   the various header types.  The menu is generated for each of the
  *   master index files.  The current header type is highlighted.
  * INPUTS
- *   dest_doc       -- the output file.
- *   document       -- the gathered documention.
- *   cur_headertype -- the header type that is to be highlighted.
+ *   * dest_doc       -- the output file.
+ *   * filename       -- the name of the output file
+ *   * document       -- the gathered documention.
+ *   * cur_headertype -- the header type that is to be highlighted.
  ******
  */
 
@@ -776,10 +866,10 @@ RB_HTML_Generate_IndexMenu(
     char type_char;
 
     assert( dest_doc );
+    assert( filename );
     assert( document );
     assert( cur_type );
 
-
     fprintf( dest_doc, "<p>\n" );
 
     for ( type_char = MIN_HEADER_TYPE;
@@ -868,21 +958,18 @@ void RB_HTML_Generate_Index_Page( struct RB_Document* document,
                 filename,
                 document,
                 header_type );
-        if ( header_type )
+        if ( RB_CompareHeaderTypes( header_type, RB_FindHeaderType( HT_SOURCEHEADERTYPE ) ) &&
+                ( header_type->typeCharacter != HT_MASTERINDEXTYPE ) )
         {
-            if ( RB_CompareHeaderTypes( header_type, RB_FindHeaderType( HT_SOURCEHEADERTYPE ) ) &&
-                 ( header_type->typeCharacter != HT_MASTERINDEXTYPE ) )
-            {
-                RB_HTML_Generate_Source_Tree( file, filename, document );
-            }
-            else
-            {
-                RB_HTML_Generate_Index_Table(
-                        file,
-                        filename,
-                        header_type,
-                        header_type->indexName );
-            }
+            RB_HTML_Generate_Source_Tree( file, filename, document );
+        }
+        else
+        {
+            RB_HTML_Generate_Index_Table(
+                    file,
+                    filename,
+                    header_type,
+                    header_type->indexName );
         }
         RB_HTML_Generate_Doc_End( file, filename );
         fclose( file );
@@ -1173,21 +1260,21 @@ RB_HTML_Generate_Empty_Item( FILE * dest_doc )
  *   RB_HTML_Generate_Link --
  * SYNOPSIS
  *   void RB_HTML_Generate_Link( FILE* dest_doc, char* dest_name, char* filename,
- *          char* labelname, char* linkname, int in_fold )
+ *   char* labelname, char* linkname, int in_fold )
  * INPUTS
- *   dest_doc  --  the file to which the text is written
- *   dest_name --  the name of the destination file
- *                 (the file from which we link)
- *   filename  --  the name of the file that contains the link
- *                 (the file we link to)
- *   labelname --  the name of the unique label of the link.
- *   linkname  --  the name of the link as shown to the user.
- *   in_fold   --  are we inside a fold.
+ *   cur_doc  --  the file to which the text is written
+ *   cur_name --  the name of the destination file
+ *                (the file from which we link)
+ *   filename --  the name of the file that contains the link
+ *                (the file we link to)
+ *   labelname--  the name of the unique label of the link.
+ *   linkname --  the name of the link as shown to the user.
+ *   in_fold  --  are we inside a fold.
  * SOURCE
  */
 
 void
-RB_HTML_Generate_Link( FILE * dest_doc, char *dest_name, char *filename,
+RB_HTML_Generate_Link( FILE * cur_doc, char *cur_name, char *filename,
                        char *labelname, char *linkname, int in_fold )
 {
     /* Include the file name in the link if we are in fold
@@ -1197,27 +1284,27 @@ RB_HTML_Generate_Link( FILE * dest_doc, char *dest_name, char *filename,
     {
         /* We are in fold, always use the file name in the link,
          * in filename == NULL (i.e. the label is in the current file
-         * that we are processing), refer to value in dest_name.
+         * that we are processing), refer to value in cur_name.
          * This also makes sure that we link correctly if linkname
          * is the same as labelname.
          */
-        fprintf( dest_doc, "<a href=\"%s#%s\">%s</a>",
-                 ( filename ? filename : dest_name ), labelname, linkname );
+        fprintf( cur_doc, "<a href=\"%s#%s\">%s</a>",
+                 ( filename ? filename : cur_name ), labelname, linkname );
     }
-    else if ( filename && strcmp( filename, dest_name ) )
+    else if ( filename && strcmp( filename, cur_name ) )
     {
         char               *r =
-            RB_HTML_RelativeAddress( dest_name, filename );
-        fprintf( dest_doc, "<a href=\"%s#%s\">", r, labelname );
-        RB_HTML_Generate_String( dest_doc, linkname );
-        fprintf( dest_doc, "</a>" );
+            RB_HTML_RelativeAddress( cur_name, filename );
+        fprintf( cur_doc, "<a href=\"%s#%s\">", r, labelname );
+        RB_HTML_Generate_String( cur_doc, linkname );
+        fprintf( cur_doc, "</a>" );
 
     }
     else
     {
-        fprintf( dest_doc, "<a href=\"#%s\">", labelname );
-        RB_HTML_Generate_String( dest_doc, linkname );
-        fprintf( dest_doc, "</a>" );
+        fprintf( cur_doc, "<a href=\"#%s\">", labelname );
+        RB_HTML_Generate_String( cur_doc, linkname );
+        fprintf( cur_doc, "</a>" );
     }
 }
 
@@ -1227,34 +1314,35 @@ RB_HTML_Generate_Link( FILE * dest_doc, char *dest_name, char *filename,
 /****f* HTML_Generator/RB_HTML_RelativeAddress
  * FUNCTION
  *   Link to 'that' from 'this' computing the relative path.  Here
- *   'this' and 'that' are both paths.
- *   This function is used to create links from one document to
- *   another document that might be in a completely different 
- *   directory.
+ *   'this' and 'that' are both paths.  This function is used to
+ *   create links from one document to another document that might be
+ *   in a completely different directory.
  * SYNOPSIS
  *   char* RB_HTML_RelativeAddress( char *thisname, char *thatname )
  * EXAMPLE
- *   this /sub1/sub2/sub3/f.html
- *   that /sub1/sub2/g.html
- *   ==
- *   ../g.html
+ *   The following two
+ *     this /sub1/sub2/sub3/f.html
+ *     that /sub1/sub2/g.html
+ *   result in 
+ *     ../g.html
  *
- *   this /sub1/f.html
- *   that /sub1/sub2/g.html
- *   ==
- *   ./sub2/g.html
+ *     this /sub1/f.html
+ *     that /sub1/sub2/g.html
+ *     ==
+ *     ./sub2/g.html
  *
- *   this /sub1/f.html
- *   that /sub1/g.html
- *   ==
- *   ./g.html
+ *     this /sub1/f.html
+ *     that /sub1/g.html
+ *     ==
+ *     ./g.html
+ *
+ *     this /sub1/doc3/doc1/tt.html
+ *     that /sub1/doc5/doc2/qq.html
+ *     ==
+ *     ../../doc5/doc2/qq.html
  *
- *   this /sub1/doc3/doc1/tt.html
- *   that /sub1/doc5/doc2/qq.html
- *   ==
- *   ../../doc5/doc2/qq.html
  * NOTES
- *   notice the execelent docmentation...
+ *   Notice the execelent docmentation.
  * SOURCE
  */
 
@@ -1300,6 +1388,7 @@ RB_HTML_RelativeAddress( char *thisname, char *thatname )
                 ++this_slashes_left;
             }
         }
+
         for ( i_c = i_that_slash + 1; *i_c; ++i_c )
         {
             if ( *i_c == '/' )
@@ -1307,6 +1396,7 @@ RB_HTML_RelativeAddress( char *thisname, char *thatname )
                 ++that_slashes_left;
             }
         }
+
         if ( this_slashes_left )
         {
             int                 i;
@@ -1317,20 +1407,17 @@ RB_HTML_RelativeAddress( char *thisname, char *thatname )
             }
             strcat( relative, i_that_slash + 1 );
         }
-        else if ( !this_slashes_left && that_slashes_left )
-        {
-            strcat( relative, "./" );
-            strcat( relative, i_that_slash + 1 );
-        }
-        else if ( !this_slashes_left && !that_slashes_left )
+        else if ( that_slashes_left )
         {
+            /* !this_slashes_left && that_slashes_left */
             strcat( relative, "./" );
             strcat( relative, i_that_slash + 1 );
         }
         else
         {
-            /* cannot get here... so it must be a BUG */
-            assert( 0 );
+            /* !this_slashes_left && !that_slashes_left */
+            strcat( relative, "./" );
+            strcat( relative, i_that_slash + 1 );
         }
     }
     return relative;
@@ -1381,7 +1468,7 @@ RB_HTML_Generate_Char( FILE * dest_doc, int c )
 
 
 
-/****f* HTML_Generator/RB_HTML_Generate_Char
+/****f* HTML_Generator/RB_Create_CSS
  * FUNCTION
  *   Create the .css file.  Unless the user specified it's own css
  *   file robodoc creates a default one.
@@ -1455,16 +1542,25 @@ RB_Create_CSS( struct RB_Document *document )
                     "    text-align: right;\n"
                     "    font-size: 11pt;\n"
                     "}\n"
+                    "td.even, td.uneven\n"
+                    "{\n"
+                    "    color: #000000;\n"
+                    "    font-size: 10pt;\n"
+                    "}\n"
                     "td.even\n"
                     "{\n"
                     "    background-color: #eeeeee;\n"
-                    "    color: #000000;\n"
-                    "    font-size: 10pt;\n"
+                    "}\n"
+                    "span.SOURCE\n"
+                    "{\n"
+                    "    white-space: pre;\n"
                     "}\n"
                     "pre\n"
                     "{\n"
                     "    background-color: #ffffff;\n"
-                    "    color: #000000;\n" "    font-size: 10pt;\n" "}\n" );
+                    "    color: #000000;\n" 
+                    "    font-size: 10pt;\n" 
+                    "}\n" );
             fclose( css_file );
         }
         else
@@ -1496,6 +1592,48 @@ RB_InsertCSS( FILE * dest_doc, char *filename )
 
 
 
+void HTML_Generate_Begin_Paragraph( FILE* dest_doc )
+{
+    fprintf( dest_doc, "<p>" );
+}
+
+void HTML_Generate_End_Paragraph( FILE* dest_doc )
+{
+    fprintf( dest_doc, "</p>\n" );
+}
+
+
+void HTML_Generate_Begin_Preformatted( FILE* dest_doc )
+{
+    fprintf( dest_doc, "<pre>" );
+}
+
+void HTML_Generate_End_Preformatted( FILE* dest_doc )
+{
+    fprintf( dest_doc, "</pre>\n" );
+}
+
+
+void HTML_Generate_Begin_List( FILE* dest_doc )
+{
+    fprintf( dest_doc, "<ul>" );
+}
+
+void HTML_Generate_End_List( FILE* dest_doc )
+{
+    fprintf( dest_doc, "</ul>\n" );
+}
+
+void HTML_Generate_Begin_List_Item( FILE* dest_doc )
+{
+    fprintf( dest_doc, "<li>" );
+}
+
+void HTML_Generate_End_List_Item( FILE* dest_doc )
+{
+    fprintf( dest_doc, "</li>\n" );
+}
+
 
 
 
