@@ -1,17 +1,17 @@
 /****h* ROBODoc/Links
  * FUNCTION
- *  This module contains functions to manipulate links.
- *  Links are derived from headers.  They are used to create
- *  links in the documentation between a word and the part of
- *  the documentation that explains something about that word.
- *  (For instance a function name or variable name).
- *  In addition to the links derived from the headers links are
- *  also derived from the names of all the sourcefiles.
+ *   This module contains functions to manipulate links.
+ *   Links are derived from headers.  They are used to create
+ *   links in the documentation between a word and the part of
+ *   the documentation that explains something about that word.
+ *   (For instance a function name or variable name).
+ *   In addition to the links derived from the headers links are
+ *   also derived from the names of all the sourcefiles.
  * MODIFICATION HISTORY
  *   ????-??-??   Frans Slothouber  V1.0 
  *   2003-02-03   Frans Slothouber  Refactoring
  *******
- * $Header: /cvsroot/robodoc/robo/Source/links.c,v 1.24 2003/12/30 17:39:36 gumpu Exp $
+ * $Header: /cvsroot/robodoc/robo/Source/links.c,v 1.31 2004/09/25 18:12:54 gumpu Exp $
  */
 
 #include <stdio.h>
@@ -34,42 +34,50 @@
 #include <dmalloc.h>
 #endif
 
+/* TODO Documentation */
+unsigned int        link_index_size = 0;
+struct RB_link    **link_index = NULL;
+struct RB_link    **case_sensitive_link_index = NULL;
+
 
 /* Local functions */
 
 static struct RB_link *RB_Alloc_Link( char *label_name, char *object_name,
                                       char *file_name );
 
-
-/* Can't get quicksort to work on Win32. So for now
- * this function is obsolete.
- */
-
-#if 0
-int
-link_cmp( const void *l1, const void *l2 )
+/* TODO Documentation */
+int link_cmp( void *l1, void *l2 )
 {
-    const struct RB_link *link1 = l1;
-    const struct RB_link *link2 = l2;
+    struct RB_link *link1 = l1;
+    struct RB_link *link2 = l2;
 
-    return strcmp( link2->label_name, link1->label_name );
+    return RB_Str_Case_Cmp( link1->object_name, link2->object_name );
 }
-#endif
+
+
+int case_sensitive_link_cmp( void *l1, void *l2 )
+{
+    struct RB_link *link1 = l1;
+    struct RB_link *link2 = l2;
+
+    return strcmp( link1->object_name, link2->object_name );
+}
+
 
 /****f* Links/RB_CollectLinks
  * FUNCTION
  *   Convert header information into link information.
- *   RB_header -> RB_link conversion
+ *      RB_header -> RB_link conversion
  * SYNOPSIS
  *   void RB_CollectLinks(struct RB_Document* document, 
- *         struct RB_header** headers, long count)
+ *   struct RB_header** headers, long count)
  * INPUTS
- *   document -- 
- *   headers  -- the array with headers.
- *   count    -- number of headers in the array
+ *   * document -- 
+ *   * headers  -- the array with headers.
+ *   * count    -- number of headers in the array
  * OUTPUT
- *   link_index -- an array with links
- *   link_index_size -- the number of links in the array.
+ *   * link_index -- an array with links
+ *   * link_index_size -- the number of links in the array.
  * SOURCE
  */
 
@@ -77,7 +85,7 @@ void
 RB_CollectLinks( struct RB_Document *document, struct RB_header **headers,
                  unsigned long count )
 {
-    unsigned int        i, j;
+    unsigned int        i;
     struct RB_Part     *i_part;
 
     link_index_size = count;
@@ -86,13 +94,19 @@ RB_CollectLinks( struct RB_Document *document, struct RB_header **headers,
     {
         for ( i_part = document->parts; i_part; i_part = i_part->next )
         {
-            link_index_size++;
+            if ( i_part->headers ) 
+            {
+                link_index_size++;
+            }
         }
     }
 
     link_index =
         ( struct RB_link ** ) calloc( link_index_size,
                                       sizeof( struct RB_link ** ) );
+    case_sensitive_link_index =
+        ( struct RB_link ** ) calloc( link_index_size,
+                                      sizeof( struct RB_link ** ) );
 
     for ( i = 0; i < count; ++i )
     {
@@ -107,6 +121,7 @@ RB_CollectLinks( struct RB_Document *document, struct RB_header **headers,
         link->htype = header->htype;
         link->is_internal = header->is_internal;
         link_index[i] = link;
+        case_sensitive_link_index[i] = link;
     }
 
     /* If we are in multidoc mode, we also create links
@@ -117,40 +132,28 @@ RB_CollectLinks( struct RB_Document *document, struct RB_header **headers,
     {
         for ( i_part = document->parts; i_part; i_part = i_part->next )
         {
-            struct RB_link     *link;
-
-            link =
-                RB_Alloc_Link( i_part->filename->name, i_part->filename->name,
-                               RB_Get_FullDocname( i_part->filename ) );
-            i_part->filename->link = link;
-//            link->type = NO_HEADER;
-            link->htype = RB_FindHeaderType( HT_SOURCEHEADERTYPE );
-            link_index[i] = link;
-            ++i;
-        }
-    }
-
-    /* Sort all the links so we can use a binary search */
-
-/* Doesn't work :( (On Win32)  qsort( link_index, link_index_size,
- * sizeof(struct RB_link*), link_cmp ); */
-
-    for ( i = 0; i < link_index_size; ++i )
-    {
-        for ( j = i + 1; j < link_index_size; ++j )
-        {
-            char               *name1 = ( link_index[i] )->object_name;
-            char               *name2 = ( link_index[j] )->object_name;
-
-            if ( strcmp( name1, name2 ) > 0 )
+            if ( i_part->headers ) 
             {
-                struct RB_link     *link = link_index[j];
+                struct RB_link     *link;
 
-                link_index[j] = link_index[i];
+                link =
+                    RB_Alloc_Link( "robo_top_of_doc", i_part->filename->name,
+                            RB_Get_FullDocname( i_part->filename ) );
+                i_part->filename->link = link;
+                link->htype = RB_FindHeaderType( HT_SOURCEHEADERTYPE );
                 link_index[i] = link;
+                case_sensitive_link_index[i] = link;
+                ++i;
+            }
+            else
+            {
+                i_part->filename->link = NULL;
             }
         }
     }
+    /* Sort all the links so we can use a binary search */
+    RB_QuickSort( link_index, 0, link_index_size - 1, link_cmp );
+    RB_QuickSort( case_sensitive_link_index, 0, link_index_size - 1, case_sensitive_link_cmp );
 }
 
 /*****/
@@ -218,38 +221,35 @@ int RB_Number_Of_Links( struct RB_HeaderType* header_type, char* file_name, int
     return n;
 }
 
-
-/****f* Links/RB_Find_Link [3.0h]
+/****f* Links/Find_Link [3.0h]
  * NAME
- *   RB_Find_Link -- try to match word with a link
+ *   Find_Link -- try to match word with a link
  * SYNOPSIS
- *   result = RB_Find_Link (word_begin, label_name, file_name)
- *   int      RB_Find_Link (char *,     char **,    char **)
+ *   int      Find_Link (char*,      char**,      char**,     char**)
  * FUNCTION
  *   Searches for the given word in the list of links and
  *   headers.  There are three passes (or four, when the C option
- *   is selected). Each pass uses a different definition of "word".
- *   In the first pass it is any thing that ends with a 'space', a '.' 
- *   or a ','.
- *   In the second pass it is any string that consists of alpha
- *   numerics, '_', ':', '.', or '-'.  
- *   In the third pass (for C) it is any string that consists 
- *   of alpha numerics or '_'.
- *   In the last pass it is any string that consists of alpha
- *   numerics.
+ *   is selected). Each pass uses a different definition of "word":
+ *   o In the first pass it is any thing that ends with a 'space', a '.' 
+ *     or a ','.
+ *   o In the second pass it is any string that consists of alpha
+ *     numerics, '_', ':', '.', or '-'.  
+ *   o In the third pass (for C) it is any string that consists 
+ *     of alpha numerics or '_'.
  * INPUTS
- *   word_begin  - pointer to a word (a string).
- *   object_name  - pointer to a pointer to a string
- *   file_name   - pointer to a pointer to a string
+ *   o word_begin  - pointer to a word (a string).
+ *   o object_name  - pointer to a pointer to a string
+ *   o file_name   - pointer to a pointer to a string
  * SIDE EFFECTS
  *   label_name & file_name are modified
  * RESULT
- *   object_name   -- points to the object if a match was found,
- *                    NULL otherwise.
- *   file_name     -- points to the file name if a match was found,
- *                    NULL otherwise.
- *   TRUE          -- a match was found.
- *   FALSE         -- no match was found.
+ *   o object_name   -- points to the object if a match was found,
+ *                      NULL otherwise.
+ *   o file_name     -- points to the file name if a match was found,
+ *                      NULL otherwise.
+ *   o label_name    -- points to the labelname if a match was found,
+ *   o TRUE          -- a match was found.
+ *   o FALSE         -- no match was found.
  * NOTES
  *   This is a rather sensitive algorithm. Don't mess with it
  *   too much.
@@ -257,25 +257,22 @@ int RB_Number_Of_Links( struct RB_HeaderType* header_type, char* file_name, int
  */
 
 int
-RB_Find_Link( char *word_begin, char **object_name, char **label_name,
+Find_Link( char *word_begin, char **object_name, char **label_name,
               char **file_name )
 {
     char               *cur_char, old_char;
     int                 low_index, high_index, cur_index, state, pass;
+    int                 length = -1;
 
-
-    for ( pass = 0; pass < 4; pass++ )
+    for ( pass = 0; pass < 3; pass++ )
     {
         switch ( pass )
         {
         case 0:
             {
                 for ( cur_char = word_begin;
-                      isalnum( *cur_char ) || ispunct( *cur_char );
+                      ( *cur_char == '_' ) || isalnum( *cur_char ) || ispunct( *cur_char );
                       cur_char++ );
-                if ( ( ( *( cur_char - 1 ) ) == ',' )
-                     || ( ( *( cur_char - 1 ) ) == '.' ) )
-                    cur_char--;
                 break;
             }
         case 1:
@@ -287,58 +284,96 @@ RB_Find_Link( char *word_begin, char **object_name, char **label_name,
                 break;
             }
         case 2:
-            {
-                if ( extra_flags & C_MODE )
-                {
-                    for ( cur_char = word_begin;
-                          isalnum( *cur_char ) || ( *cur_char == '_' );
-                          cur_char++ );
-                    break;
-                }
-                else
-                    continue;
-            }
-        case 3:
             {
                 for ( cur_char = word_begin;
-                      isalnum( *cur_char ); cur_char++ );
+                      isalnum( *cur_char ) || ( *cur_char == '_'); 
+                      cur_char++ );
                 break;
             }
         }
 
+        if ( ( ( *( cur_char - 1 ) ) == ',' ) || ( ( *( cur_char - 1 ) ) == '.' ) )
+        {
+            cur_char--;
+        }
+
         old_char = *cur_char;
         *cur_char = '\0';       /*
-                                   * End the word with a '\0' 
+                                 * End the word with a '\0' 
                                  */
-        /*
-         * RB_Say ("Testing \"%s\"\n", word_begin); 
-         */
 
-        /*
-         * Search in the link table 
-         */
-        for ( cur_index = 0, low_index = 0, high_index =
-              link_index_size - 1; high_index >= low_index; )
+
+        if ( strlen( word_begin ) == length )
         {
-            cur_index = ( high_index - low_index ) / 2 + low_index;
-            state = strcmp( word_begin, link_index[cur_index]->object_name );
-            if ( state < 0 )
+            /* Do not test the same word over and over. If
+             * the current string and the string of the previous
+             * pass are the same length, they are the same. */
+
+            /* RB_Say ("Skipping (pass %d) \"%s\"\n", pass, word_begin);  */
+        }
+        else
+        {
+            length = strlen( word_begin );
+            /* RB_Say ("Testing (pass %d) \"%s\"\n", pass, word_begin); */
+            /*
+             * Search case sensitive for a link 
+             */
+            for ( cur_index = 0, low_index = 0, high_index =
+                    link_index_size - 1; high_index >= low_index; )
             {
-                high_index = cur_index - 1;
+                cur_index = ( high_index - low_index ) / 2 + low_index;
+                state = strcmp( word_begin, case_sensitive_link_index[cur_index]->object_name );
+                if ( state < 0 )
+                {
+                    high_index = cur_index - 1;
+                }
+                else if ( state == 0 )
+                {
+                    *object_name = case_sensitive_link_index[cur_index]->object_name;
+                    *label_name = case_sensitive_link_index[cur_index]->label_name;
+                    *file_name = case_sensitive_link_index[cur_index]->file_name;
+                    RB_Say( "linking \"%s\"->\"%s\" from \"%s\"\n",
+                            word_begin, *object_name, *file_name );
+                    *cur_char = old_char;
+                    return ( TRUE );
+                }
+                else if ( state > 0 )
+                {
+                    low_index = cur_index + 1;
+                }
             }
-            else if ( state == 0 )
+
+            /*
+             * Search case insensitive for a link.
+             * But only when the user asks for this.
+             */
+            if ( course_of_action & DO_IGNORE_CASE_WHEN_LINKING ) 
             {
-                *object_name = link_index[cur_index]->object_name;
-                *label_name = link_index[cur_index]->label_name;
-                *file_name = link_index[cur_index]->file_name;
-                RB_Say( "linking \"%s\"->\"%s\" form \"%s\"\n",
-                        word_begin, *object_name, *file_name );
-                *cur_char = old_char;
-                return ( TRUE );
-            }
-            else if ( state > 0 )
-            {
-                low_index = cur_index + 1;
+
+                for ( cur_index = 0, low_index = 0, high_index =
+                        link_index_size - 1; high_index >= low_index; )
+                {
+                    cur_index = ( high_index - low_index ) / 2 + low_index;
+                    state = RB_Str_Case_Cmp( word_begin, link_index[cur_index]->object_name );
+                    if ( state < 0 )
+                    {
+                        high_index = cur_index - 1;
+                    }
+                    else if ( state == 0 )
+                    {
+                        *object_name = link_index[cur_index]->object_name;
+                        *label_name = link_index[cur_index]->label_name;
+                        *file_name = link_index[cur_index]->file_name;
+                        RB_Say( "linking \"%s\"->\"%s\" from \"%s\"\n",
+                                word_begin, *object_name, *file_name );
+                        *cur_char = old_char;
+                        return ( TRUE );
+                    }
+                    else if ( state > 0 )
+                    {
+                        low_index = cur_index + 1;
+                    }
+                }
             }
         }
         *cur_char = old_char;
@@ -424,3 +459,4 @@ RB_Free_Link( struct RB_link *link )
 }
 
 /******/
+
