@@ -9,14 +9,14 @@
  *  NAME
  *    RB_link -- link data structure
  *  PURPOSE
- *    Structure to store links to the documentation of an component. 
- *  PROPERTIES
- *    label_name  -- the label under which the component can be found.
- *                   this should be a unique name.
- *    object_name -- the proper name of the object
- *    file_name   -- the file the component can be found in.
- *    type        -- the type of component (the header type).
- *    is_internal -- is the header an internal header?
+ *    Structure to store links to the documentation of an component.
+ *  ATTRIBUTES
+ *    * label_name  -- the label under which the component can be found.
+ *                     this should be a unique name.
+ *    * object_name -- the proper name of the object
+ *    * file_name   -- the file the component can be found in.
+ *    * type        -- the type of component (the header type).
+ *    * is_internal -- is the header an internal header?
  *  SOURCE
  */
 
@@ -31,7 +31,7 @@ struct RB_link
 
 /*********/
 
-int                 RB_Find_Link( char *word_begin, char **object_name,
+int                 Find_Link( char *word_begin, char **object_name,
                                   char **unique_name, char **file_name );
 void                RB_CollectLinks( struct RB_Document *document, struct RB_header **headers, unsigned long count );
 void                RB_Free_Links( void );
