@@ -6,7 +6,7 @@
  *   The robodoc.rc file consists of a number of blocks.  Each
  *   block starts with a line of the form 
  *
- *   <block name>:
+ *     <block name>:
  *
  *   This is followed by a number of lines of data.  Each line starts
  *   with at least one space followed by the actual data.
@@ -18,7 +18,7 @@
  *   Is missing a lot of documentation.
  *
  ******
- * $Id: roboconfig.c,v 1.23 2003/12/30 17:39:36 gumpu Exp $
+ * $Id: roboconfig.c,v 1.30 2004/06/16 20:31:49 gumpu Exp $
  */
 
 #include <stdio.h>
@@ -95,28 +95,22 @@ struct RB_Configuration configuration;
 
 /*****/
 
-static void         RB_AllocItemNames( void );
 static void         RB_AllocOptions( unsigned int argc, char **argv );
 static T_Block_Kind RB_BlockKind( char *line );
 static T_Line_Kind  RB_ConfigLineKind( char *line );
-static void         RB_FirstScan( FILE * f );
 static void         RB_SecondScan( FILE * f );
-static void         RB_AllocIgnoreItems( void );
 static void         RB_Alloc_Parameters( struct Parameters* parameters, unsigned int size );
 static void         RB_AddParameter( char* name, struct Parameters* parameters );
 static void         RB_GetParameters( char* line, struct Parameters* parameters );
 static void         RB_Install_Custom_HeaderTypes( void );
+static void         RB_ComplementItemNames( void );
+static void         RB_ComplementHeaderMarkers( void );
 
 
 /****f* Configuration/RB_ReadConfiguration
  * FUNCTION
  *   Read the robodoc configuration file, and create
  *   a RB_Configuration structure.
- *
- *   The configuration is scanned twice. The first scan
- *   is to determine the number of elements in each section,
- *   in the second scan the elements are read and stored.
- *
  * SYNOPSIS
  *   void RB_ReadConfiguration( unsigned int argc, char **argv, char* filename )
  * INPUTS
@@ -125,7 +119,7 @@ static void         RB_Install_Custom_HeaderTypes( void );
  *   o filename -- an optional filename.  If none is given,
  *               "robodoc.rc" is used.
  * RESULT
- *   An initialized configuration.
+ *   An initialized configuration (a global).
  * SOURCE
  */
 
@@ -148,27 +142,80 @@ RB_ReadConfiguration( unsigned int argc, char **argv, char* filename )
         /* TODO look at other locations. */
     }
 
-    if ( f )
-    {
-        RB_FirstScan( f );
-    }
-
-    RB_AllocItemNames();
     RB_AllocOptions( argc, argv );
-    RB_AllocIgnoreItems();
+    RB_Alloc_Parameters( &( configuration.items ), 10 );
+    RB_Alloc_Parameters( &( configuration.ignore_items ), 10 );
     RB_Alloc_Parameters( &( configuration.custom_headertypes ), 10 );
     RB_Alloc_Parameters( &( configuration.ignore_files ), 10 );
+    RB_Alloc_Parameters( &( configuration.accept_files ), 10 );
+    RB_Alloc_Parameters( &( configuration.header_markers ), 10 );
+    RB_Alloc_Parameters( &( configuration.remark_markers ), 10 );
+    RB_Alloc_Parameters( &( configuration.end_markers ), 10 );
     if ( f )
     {
-        rewind( f );
         RB_SecondScan( f );
         fclose( f );
     }
+    else
+    {
+        /* No .rc file found.  That's OK */
+    }
+    RB_ComplementItemNames();
+    RB_ComplementHeaderMarkers();
     RB_Install_Custom_HeaderTypes();
+
+    assert( configuration.items.number );
 }
 
 /******/
 
+/* TODO Documentation */
+static void RB_ComplementHeaderMarkers( void )
+{
+    unsigned int  i;
+
+    if ( configuration.header_markers.number )
+    {
+        /* The user specified his own header_markers */
+    }
+    else
+    {
+        for( i = 0; header_markers[ i ]; ++i )
+        {
+            RB_Say("Adding header marker (%s)\n", header_markers[ i ]);
+            RB_AddParameter( header_markers[ i ], &( configuration.header_markers ) );
+        }
+    }
+
+    if ( configuration.remark_markers.number )
+    {
+        /* The user specified his own remark_markers */
+    }
+    else
+    {
+        for( i = 0; remark_markers[ i ]; ++i )
+        {
+            RB_Say("Adding remark marker (%s)\n", remark_markers[ i ]);
+            RB_AddParameter( remark_markers[ i ], &( configuration.remark_markers ) );
+        }
+    }
+
+    if ( configuration.end_markers.number )
+    {
+        /* The user specified his own end_markers */
+    }
+    else
+    {
+        for( i = 0; end_markers[ i ]; ++i )
+        {
+            RB_Say("Adding end marker (%s)\n", end_markers[ i ]);
+            RB_AddParameter( end_markers[ i ], &( configuration.end_markers ) );
+        }
+    }
+}
+
+
+
 /****if* Config/RB_ConfigLineKind
  * FUNCTION
  *   Deterimine the kind of line we a currently processing.
@@ -238,7 +285,7 @@ RB_BlockKind( char *line )
     else if ( strcmp( line, "extensions:" ) == 0 )
     {
         section_kind = SK_EXTENSIONS;
-        printf("Warning:  the 'extensions:' section is obsolete, use 'ignore files:' instead\n");
+        printf("Warning:  the 'extensions:' block is obsolete, use 'ignore files:' instead\n");
     }
     else if ( strcmp( line, "ignore items:" ) == 0 )
     {
@@ -252,9 +299,25 @@ RB_BlockKind( char *line )
     {
         section_kind = SK_IGNORE_FILES;
     }
+    else if ( strcmp( line, "accept files:" ) == 0 )
+    {
+        section_kind = SK_ACCEPT_FILES;
+    }
+    else if ( strcmp( line, "header markers:") == 0 )
+    {
+        section_kind = SK_HEADER_MARKERS;
+    }
+    else if ( strcmp( line, "remark markers:") == 0 )
+    {
+        section_kind = SK_REMARK_MARKERS;
+    }
+    else if ( strcmp( line, "end markers:") == 0 )
+    {
+        section_kind = SK_END_MARKERS;
+    }
     else
     {
-        RB_Panic( "unknown section kind \"%s\"\n", line );
+        RB_Panic( "unknown block kind \"%s\"\n", line );
     }
     return section_kind;
 }
@@ -306,7 +369,19 @@ static void RB_AddParameter( char* name, struct Parameters* parameters )
 }
 
 
-/* TODO Documentation */
+/****f* Configuration/RB_GetParameters
+ * FUNCTION
+ *   Parse a line of text and store the individual words in 
+ *   a Parameters structure.  Words are seperated by spaces,
+ *   the exception are words surrounded by quotes. So:
+ *      aap noot mies "back to the future"
+ *   contains four words.
+ * INPUTS
+ *   o line -- the line of text.
+ *   o parameters  -- the set of parameters
+ * SOURCE
+ */
+
 static void RB_GetParameters( char* line, struct Parameters* parameters )
 {
     int                 i;
@@ -390,10 +465,38 @@ static void RB_GetParameters( char* line, struct Parameters* parameters )
     }
 }
 
+/*******/
+
+void 
+RB_GetParameter( char *line, struct Parameters* parameters )
+{
+    int                 i;
+    int                 n = strlen( line );
+
+    /* Remove any spaces at the end of the line */
+    for ( i = n - 1; i >= 0 && isspace( line[i] ); --i )
+    {
+        line[i] = '\0';
+    }
+    assert( i > 0 );            /* If i <= 0 then the line was empty
+                                   and that cannot be, because this 
+                                   is supposed to be a parameter */
+    /* Skip any white space at the begin of the line. */
+    n = strlen( line );
+    for ( i = 0; i < n && isspace( line[i] ); ++i )
+    {
+        /* Empty */
+    }
+    line += i;
+
+    RB_AddParameter( line, parameters );
+}
 
 
-char               *
-RB_GetParameter( char *line )
+
+
+char*
+RB_GetParameter_Old( char *line )
 {
     int                 i;
     int                 n = strlen( line );
@@ -421,58 +524,14 @@ RB_GetParameter( char *line )
 }
 
 
-void
-RB_StripCR( char *line )
-{
-    char               *c;
-
-    for ( c = line; *c; ++c )
-    {
-        if ( *c == '\n' )
-        {
-            *c = '\0';
-        }
-    }
-}
-
 void
 RB_Free_Configuration( void  )
 {
-    unsigned int        i;
-
-    for ( i = 0; i < configuration.no_items; ++i )
-    {
-        free( configuration.item_names[i] );
-    }
-    free( configuration.item_names );
-
-    for ( i = 0; i < configuration.no_ignore_items; ++i )
-    {
-        free( configuration.ignore_items[i] );
-    }
-    free( configuration.ignore_items );
     /* TODO  Deallocate custom_headertypes */
 }
 
 
 
-/* TODO Documentation */
-static void
-RB_AllocIgnoreItems( void )
-{
-    if ( configuration.no_ignore_items )
-    {
-        configuration.ignore_items =
-            ( char ** ) calloc( configuration.no_ignore_items,
-                                sizeof( char * ) );
-    }
-    else
-    {
-        configuration.ignore_items = NULL;
-    }
-}
-
-
 /* TODO Documentation */
 static void
 RB_AllocOptions( unsigned int argc, char **argv )
@@ -486,88 +545,25 @@ RB_AllocOptions( unsigned int argc, char **argv )
 }
 
 
-
-
-/* TODO Documentation */
-static void
-RB_AllocItemNames( void )
+static void RB_ComplementItemNames( void )
 {
-    if ( configuration.no_items )
+    if ( configuration.items.number )
     {
-        configuration.no_items++;
-        configuration.item_names =
-            ( char ** ) calloc( configuration.no_items, sizeof( char * ) );
-        assert( configuration.item_names );
-        /* The SOURCE item is always included */
-        configuration.item_names[0] = RB_StrDup( "SOURCE" );
+        char* first_item = configuration.items.names[ 0 ];
+        /* The SOURCE item is always included, and should be the
+         * first one! */
+        configuration.items.names[ 0 ] = RB_StrDup( "SOURCE" );
+        RB_AddParameter( first_item, &( configuration.items ) );
+        free( first_item );
     }
     else
     {
         /* No item names were defined, so we use the default ones */
-        unsigned int                 i;
-
-        for ( ; default_item_names[configuration.no_items];
-              ++configuration.no_items )
+        unsigned int                 i = 0;
+        for ( ; default_item_names[ i ]; ++i )
         {
-            /* empty */
-        }
-        configuration.item_names =
-            ( char ** ) calloc( configuration.no_items, sizeof( char * ) );
-        for ( i = 0; i < configuration.no_items; ++i )
-        {
-            configuration.item_names[i] = RB_StrDup( default_item_names[i] );
-        }
-    }
-    RB_Say( "There are %d item names\n", configuration.no_items );
-}
-
-
-
-/* Count number of paramters and options */
-static void
-RB_FirstScan( FILE * f )
-{
-    char                line_buffer[BUFFER_LENGTH];
-    T_Block_Kind      section_kind = SK_UNKNOWN;
-    T_Line_Kind         line_kind = CFL_UNKNOWN;
-
-    while ( fgets( line_buffer, BUFFER_LENGTH, f ) )
-    {
-        RB_StripCR( line_buffer );
-        line_kind = RB_ConfigLineKind( line_buffer );
-        switch ( line_kind )
-        {
-        case CFL_REMARK:
-        case CFL_EMPTYLINE:    /* fall through */
-            break;
-        case CFL_SECTION:
-            section_kind = RB_BlockKind( line_buffer );
-            break;
-        case CFL_PARAMETER:
-            {
-                switch ( section_kind )
-                {
-                case SK_ITEMS:
-                    ++configuration.no_items;
-                    break;
-                case SK_EXTENSIONS:
-                    /* Obsolete */
-                    break;
-                case SK_IGNOREITEMS:
-                    ++configuration.no_ignore_items;
-                    break;
-                case SK_OPTIONS:      /* fall through */
-                case SK_HEADERTYPES: 
-                case SK_IGNORE_FILES:
-                    break;
-                case SK_UNKNOWN:
-                    assert( 0 );
-                }
-            }
-            break;
-        case CFL_UNKNOWN:
-        default:
-            assert( 0 );
+            RB_Say( "Adding item %s", default_item_names[ i ] );
+            RB_AddParameter( default_item_names[ i ], &( configuration.items ) );
         }
     }
 }
@@ -580,8 +576,6 @@ RB_SecondScan( FILE * f )
     char                line_buffer[BUFFER_LENGTH];
     T_Block_Kind      section_kind = SK_UNKNOWN;
     T_Line_Kind         line_kind = CFL_UNKNOWN;
-    int                 itemname_nr = 1;        /* 0 if for the SOURCE_ITEM */
-    int                 ignore_nr = 0;
 
     while ( fgets( line_buffer, BUFFER_LENGTH, f ) )
     {
@@ -600,32 +594,40 @@ RB_SecondScan( FILE * f )
             {
                 switch ( section_kind )
                 {
-                case SK_ITEMS:
-                    configuration.item_names[itemname_nr] =
-                        RB_GetParameter( line_buffer );
-                    ++itemname_nr;
-                    break;
-                case SK_OPTIONS:
-                    RB_GetParameters( line_buffer, &( configuration.options ) );
-                    break;
-                case SK_EXTENSIONS:
-                    /* Obsolete */
-                    break;
-                case SK_IGNOREITEMS:
-                    configuration.ignore_items[ignore_nr] =
-                        RB_GetParameter( line_buffer );
-                    ++ignore_nr;
-                    break;
-                case SK_HEADERTYPES:
-                    RB_GetParameters( line_buffer, &( configuration.custom_headertypes ) );
-                    break;
-                case SK_IGNORE_FILES:
-                    RB_GetParameters( line_buffer, &( configuration.ignore_files ) );
-                    break;
-                case SK_UNKNOWN:
-                    break;
-                default:
-                    assert( 0 );
+                    case SK_ITEMS:
+                        RB_GetParameter( line_buffer, &( configuration.items ) );
+                        break;
+                    case SK_OPTIONS:
+                        RB_GetParameters( line_buffer, &( configuration.options ) );
+                        break;
+                    case SK_EXTENSIONS:
+                        /* Obsolete */
+                        break;
+                    case SK_IGNOREITEMS:
+                        RB_GetParameter( line_buffer, &( configuration.ignore_items ) );
+                        break;
+                    case SK_HEADERTYPES:
+                        RB_GetParameters( line_buffer, &( configuration.custom_headertypes ) );
+                        break;
+                    case SK_IGNORE_FILES:
+                        RB_GetParameters( line_buffer, &( configuration.ignore_files ) );
+                        break;
+                    case SK_ACCEPT_FILES:
+                        RB_GetParameters( line_buffer, &( configuration.accept_files ) );
+                        break;
+                    case SK_HEADER_MARKERS :
+                        RB_GetParameter( line_buffer, &( configuration.header_markers ) );
+                        break;
+                    case SK_REMARK_MARKERS :
+                        RB_GetParameter( line_buffer, &( configuration.remark_markers ) );
+                        break;
+                    case SK_END_MARKERS :
+                        RB_GetParameter( line_buffer, &( configuration.end_markers ) );
+                        break;
+                    case SK_UNKNOWN:
+                        break;
+                    default:
+                        assert( 0 );
                 }
             }
             break;
